package ie.ucd.csi.comp10050.cellular_automata;

/**
 * @explanation The smallest unit of value in a cellular automata.
 *
 * @author Joe Kiniry (kiniry@ucd.ie)
 * @module COMP 10050
 * @organisation School of Computer Science and Informatics
 * @copyright University College Dublin
 * @version 5 April 2006
 */

public abstract class Cell {
  /**
   * @design kiniry 10 March 2006 - We use longs for the values
   * of cells for the following reasons:
   * <ul>
   *   <li> precise representation (no fuzzyness of floating point) </li>
   *   <li> easy comparison </li>
   *   <li> their values are a total order </li>
   *   <li> there is a direct connection with the decimal representation
   *        of rules, lengths, etc. </li>
   * </ul>
   *
   * @design kiniry 10 March 2006 - A cell's values are not necessarily
   * consecutive.  E.g., perhaps your valid values are 1, 10, and 20.
   *
   * @design kiniry 10 March 2006 - Valid values are always positive.
   */

  // @bon query

  /**
   * @param the_value the value to check.
   * @return Is <var>the_value</var> a valid value?
   */
  //@ requires 0 <= the_value;
  public abstract /*@ pure @*/ boolean validValue(final long the_value);

  /**
   * @return How many valid values are there?
   */
  //@ ensures 0 <= \result & \result < Long.MAX_VALUE;
  public abstract /*@ pure @*/ long validValueCount();

  /**
   * @return What is the textual representation of this cell?
   */
  public abstract /*@ pure @*/ char textualRepresentation();

  /**
   * @return What is your value?
   */
  //@ ensures validValue(\result);
  public abstract /*@ pure @*/ long value();

  /**
   * @param a_cell the cell against which to compare.
   * @return Is this cell equivalent to the cell <var>a_cell</var>?
   *
   * @note This method overrides the method {@link
   * java.lang.Object#equals(java.lang.Object)}.
   */
  //@ also
  //@ requires a_cell instanceof Cell;
  //@ ensures \result <==> (value() == ((Cell)a_cell).value());
  public abstract /*@ pure @*/ boolean equals(/*@ nullable */ final Object a_cell);

  // @bon command

  /**
   * Set the value of this cell to <var>the_value</var>.
   *
   * @param the_value the new value for this cell.
   */
  //@ requires validValue(the_value);
  //@ ensures value() == the_value;
  public abstract void setValue(final long the_value);

  // @bon constraint

  // @bon Different cells have different values.
  //@ public invariant (\forall Cell c, d; ; !c.equals(d) <==> c.value() != d.value());

  // @bon Different cells have different textual representation.
  /*@ public invariant (\forall Cell c, d; ;
    @   !c.equals(d) <==> c.textualRepresentation() != d.textualRepresentation());
    @*/
}
