package ie.ucd.csi.comp10050.cellular_automata;

/**
 * A one-dimensional cellular automata simulator.
 *
 * @author Joe Kiniry (kiniry@ucd.ie)
 * @module COMP 10050
 * @organisation School of Computer Science and Informatics
 * @copyright University College Dublin
 * @version 5 April 2006
 */

public abstract class CellularAutomata {
  // @bon query

  /**
   * @return What is current length?
   */
  // @constraint At any time there is exactly one length.
  public abstract /*@ pure non_null @*/ Length length();

  /**
   * @return How many rules are there in this automata?
   */
  //@ ensures 0 <= \result;
  public abstract /*@ pure @*/ int numberOfRules();

  /**
   * @param a_rule_number the number of the rule we are interested in.
   * @return What is rule <var>rule_number</var>?
   */
  //@ requires 0 <= a_rule_number && a_rule_number < numberOfRules();
  public abstract /*@ pure non_null @*/ Rule getRule(final int a_rule_number);

  /**
   * @return What is the current time step?
   */
  // @constraint Time is always non-negative.
  //@ ensures 0 <= \result;
  public abstract /*@ pure @*/ int timeStep();

  /**
   * @param a_cellular_automata the cellular automata against which to compare.
   * @return Is this automata equivalent to the automata <var>a_cellularautomata</var>?
   */
  public abstract /*@ pure @*/ boolean equals(final /*@ nullable */ Object a_cellular_automata);

  // @bon command

  /**
   * Set rule <code>rule_number</code> to <code>rule</code>.
   *
   * @param the_rule_number the rule number to change.
   * @param the_new_rule the new rule.
   */
  //@ requires 0 <= the_rule_number && the_rule_number < numberOfRules();
  //@ ensures getRule(the_rule_number).equals(the_new_rule);
  public abstract void setRule(final int the_rule_number,
                               final /*@ non_null @*/ Rule the_new_rule);

  /**
   * Set the current length.
   *
   * @param a_length the new length of the simulator.
   */
  //@ ensures length().equals(a_length);
  public abstract void setLength(final /*@ non_null @*/ Length a_length);

  /**
   * Set the current time step to <var>a_timestep</var>.
   *
   * @param a_timestep the new time step for this simulator.
   */
  //@ requires 0 <= a_timestep;
  //@ ensures timeStep() == a_timestep;
  public abstract void setTimeStep(final int a_timestep);

  /**
   * Increments the current time step by one.
   */
  //@ ensures timeStep() == \old(timeStep() + 1);
  public abstract void nextTimeStep();

  /**
   * Simulate this automata starting for <var>some_timesteps</var>
   * time steps.
   *
   * @param some_timesteps the number of time steps to simulate.
   */
  //@ requires 0 <= some_timesteps;
  public abstract void simulate(final int some_timesteps);

  /**
   * Simulate this automata starting for <var>some_timesteps</var>
   * time steps, printing out each length to the stream
   * <var>output</var>.  If <var>output</var> is <code>null</code>,
   * then print no output.
   *
   * @param some_timesteps the number of time steps to simulate.
   * @param the_output the stream to which we output the simulation.
   */
  //@ requires 0 <= some_timesteps;
  public abstract void simulate(final int some_timesteps,
                                final java.io.PrintStream the_output);

  // @constraint Time starts at 0.
  //@ public initially timeStep() == 0;

  // @constraint The relationship between a length at time t0 and a
  // length at time t1 is completely determined by the rules of this
  // cellular automata.
}
