package ie.ucd.csi.comp10050.cellular_automata;

/**
 * A cell whose value is one of a specific set of values.
 *
 * @author Joe Kiniry (kiniry@ucd.ie)
 * @module COMP 10050
 * @organisation School of Computer Science and Informatics
 * @copyright University College Dublin
 * @version 5 April 2006
 *
 * @design kiniry 10 March 2006 - Indexed cells have a finite set of
 * values, that are interpreted as, e.g., flavours, colours, etc.
 */

public abstract class IndexedCell extends Cell {
  // @bon query

  /**
   * The maximum number of values that an <var>IndexedCell</var> may have.
   */
  public static final int MAXIMUM_INDEX = 1024;

  /** {@inheritDoc} */
  // @constraint An indexed cell's values are consecutive and start with zero.
  //@ also
  //@ ensures \result <==> 0 <= the_value && the_value < indexSize();
  public abstract /*@ pure @*/ boolean validValue(final long the_value);

  /** {@inheritDoc} */
  //@ also
  //@ ensures 0 <= \result & \result <= MAXIMUM_INDEX;
  public abstract /*@ pure @*/ long validValueCount();

  /**
   * @return the size of the index set for this indexed cell.
   */
  //@ ensures \result == validValueCount();
  public abstract /*@ pure @*/ int indexSize();

  /** {@inheritDoc} */
  //@ also
  //@ requires a_cell instanceof IndexedCell;
  //@ ensures \result <==> (validValueCount() == ((IndexedCell)a_cell).validValueCount());
  public abstract /*@ pure @*/ boolean equals(final /*@ nullable */ Object a_cell);

  // @bon command

  /**
   * Set the number of valid values for this indexed cell to
   * <var>indexSize</var>.
   *
   * @param an_indexsize the number of different values that this cell
   * can have.
   */
  //@ requires 0 <= an_indexsize & an_indexsize <= MAXIMUM_INDEX;
  //@ ensures indexSize() == an_indexsize;
  public abstract void setIndexSize(final int an_indexsize);

  // @bon constraint
}
