package ie.ucd.csi.comp10050.cellular_automata;

/**
 * A finite sequence of cells.
 *
 * @author Joe Kiniry (kiniry@ucd.ie)
 * @module COMP 10050
 * @organisation School of Computer Science and Informatics
 * @copyright University College Dublin
 * @version 5 April 2006
 */

public abstract class Length {
  // @design kiniry 10 Mar 2006 - We choose to represent positions
  // with Java ints because Java arrays are indexed by ints and
  // we want a simple representation for Lengths, Rules, etc.

  // @design kiniry 10 Mar 2006 - Note the lack of constructors
  // in these classes.  Abstract classes cannot have constructors.
  // Your subclasses should just use *default* constructors.

  // @bon query

  /**
   * @return How many positions do you have?
   */
  //@ ensures 0 <= \result;
  public abstract /*@ pure @*/ int size();

  /**
   * @param the_position the position we are interested in.
   * @return Which cell is at position <var>thePosition</var>?
   */
  //@ requires 0 <= the_position && the_position < size();
  public abstract /*@ pure non_null @*/ Cell getCell(final long the_position);

  /**
   * @return What is your decimal representation?
   */
  public abstract /*@ pure @*/ long decimalRepresentation();

  /**
   * @param a_length the length against which to compare.
   * @return Is this length equivalent to the length <var>aLength</var>?
   */
  public abstract boolean equals(final /*@ nullable */ Object a_length);

  // @bon command

  /**
   * Set position <var>a_position</var> to cell <var>a_cell</var> in this length.
   *
   * @param a_cell the cell to put into this length.
   * @param a_position the position at which <var>a_cell</var> must be inserted.
   */
  //@ requires 0 <= a_position && a_position < size();
  public abstract void setCell(final /*@ non_null @*/ Cell a_cell, final int a_position);

  /**
   * Set this length to <var>the_representation</var>.
   *
   * @param the_representation the new representation for this length.
   */
  public abstract void setDecimalRepresentation(final long the_representation);

  // @constraint All cells in a length are of the same type.
  //@ public invariant (\forall Cell c, d; c != null && d != null; \typeof(c) == \typeof(d));
}
