package ie.ucd.csi.comp10050.cellular_automata;

/**
 * The rule which defines a cellular automata's behavior.
 *
 * @author Joe Kiniry (kiniry@ucd.ie)
 * @module COMP 10050
 * @organisation School of Computer Science and Informatics
 * @copyright University College Dublin
 * @version 5 April 2006
 */

public abstract class Rule {
  // @bon query

  /**
   * @return What is your start state?
   */
  public abstract /*@ pure non_null @*/ Length startState();

  /**
   * @return What is your final state?
   */
  public abstract /*@ pure non_null @*/ Length finalState();

  /**
   * @return Which cells must be updated?
   */
  public abstract /*@ pure non_null @*/ Length cellsToUpdate();

  /**
   * @param a_rule the rule against which to compare.
   * @return Is this rule equivalent to the rule <var>aRule</var>?
   */
  public abstract boolean equals(final /*@ nullable */ Object a_rule);

  // @bon command

  /**
   * Set the start state to <var>a_state</var>.
   *
   * @param the_new_state the new start state.
   */
  //@ ensures startState().equals(the_new_state);
  public abstract void setStartState(final /*@ non_null @*/ Length the_new_state);

  /**
   * Set the final state to <var>the_new_state</var>.
   *
   * @param the_new_state the new final state.
   */
  //@ ensures finalState().equals(the_new_state);
  public abstract void setFinalState(final /*@ non_null @*/ Length the_new_state);

  /**
   * Designate which cells to update.
   *
   * @param a_cell_designation a length whose cells indicate which cells
   * are to be updated by this rule.
   */
  //@ requires a_cell_designation.size() == startState().size();
  //@ ensures cellsToUpdate().equals(a_cell_designation);
  public abstract void setCellsToUpdate(final /*@ non_null @*/ Length a_cell_designation);

  // @bon contraint
}
